<?php
class flowCommon extends commonModel
{
    /**
     * Merge lang from flow.
     *
     * @access public
     * @return void
     */
    public function mergeLangFromFlow()
    {
        if(defined('IN_UPGRADE')) return;
        if(!$this->config->db->name) return;

        $flowApp    = '';
        $rawModule  = $this->app->rawModule;
        $flows      = $this->dao->select('*')->from(TABLE_WORKFLOW)->orderBy('buildin_desc, navigator_asc')->fetchAll('id');
        $flowFields = $this->loadModel('workflowfield')->getGroupList();
        $flowLabels = $this->loadModel('workflowlabel')->getGroupList();
        $flowOrder  = $this->sortFlows($flows);

        foreach($flowOrder as $flowID)
        {
            $flow = $flows[$flowID];

            /* Init lang var of module, module menu. */
            if(!isset($this->lang->{$flow->module}))             $this->lang->{$flow->module}             = new stdclass();
            if(!isset($this->lang->{$flow->module}->menu))       $this->lang->{$flow->module}->menu       = new stdclass();
            if(!isset($this->lang->{$flow->module}->featureBar)) $this->lang->{$flow->module}->featureBar = array();

            $fields = zget($flowFields, $flow->module, array());
            foreach($fields as $field)
            {
                if(isset($this->lang->{$flow->module}->{$field->field})) continue;
                $this->lang->{$flow->module}->{$field->field} = $field->name;
            }

            if($flow->parent or $flow->type != 'flow') continue;

            if($flow->module == $rawModule) $flowApp = $flow->app;

            if($flow->navigator == 'primary')
            {
                /* Get default main menu. */
                $menu = $this->lang->menu;
            }
            elseif($flow->buildin)
            {
                /* Get default module menu. */
                $menu = isset($this->lang->{$flow->app}->menu) ? $this->lang->{$flow->app}->menu : $this->lang->menu;
                if(isset($this->lang->{$rawModule}->menu)) $menu = $this->lang->{$rawModule}->menu;
            }
            else
            {
                if(!isset($this->lang->{$flow->app}->menu)) $this->lang->{$flow->app}->menu = new stdclass();
                $menu = $this->lang->{$flow->app}->menu;

                if(isset($this->lang->menugroup->$rawModule) and $this->lang->menugroup->$rawModule == $flow->app)
                {
                    if(isset($this->lang->{$rawModule}->menu)) $menu = $this->lang->{$rawModule}->menu;
                }

                if(!isset($this->lang->menugroup->{$flow->module})) $this->lang->menugroup->{$flow->module} = $flow->app;
            }

            /* Get default feature bar. */
            $featureBar = $this->getFeatureBar($flow);

            if(isset($this->app->user) && (isset($this->app->user->rights['rights'][$flow->module]) or $this->app->user->admin))
            {
                $labels = zget($flowLabels, $flow->module, array());
                foreach($labels as $label)
                {
                    if($label->buildin) continue;
                    if(!commonModel::hasPriv($flow->module, $label->id)) continue;

                    if(($flow->buildin && $featureBar) or !$flow->buildin)
                    {
                        /* Set feature bar. */
                        if($flow->buildin)
                        {
                            $menuItem = array();
                            $menuItem['label'] = $label->label;
                            $menuItem['link']  = helper::createLink($flow->module, 'browseLabel', "mode=browse&label=$label->id");
                        }
                        else
                        {
                            $menuItem = "$label->label|$flow->module|browse|mode=browse&label=$label->id";
                        }
                        if(is_object($featureBar) and !isset($featureBar->{$label->code})) $featureBar->{$label->code} = $menuItem;
                        if(is_array($featureBar) and !isset($featureBar[$label->code]))    $featureBar[$label->code]   = $menuItem;
                    }

                    $menuItem = "%s|$flow->module|browse|mode=browse&label=$label->id";

                    /* If flow's navigator is primary, add flow to main menu. */
                    /* If flow's navigator is secondary, add flow to module menu. */
                    if(!isset($menu->{$flow->module})) $menu->{$flow->module} = sprintf($menuItem, $flow->name); 

                    if($flow->navigator == 'primary')
                    {
                        $this->lang->{$flow->module}->menuOrder[] = $label->code;
                        /* Add label to module menu. */
                        $this->lang->{$flow->module}->menu->{$label->code} = sprintf($menuItem, $label->label);
                    }
                }
                /* Add search tab to module menu. */
                if(!$flow->buildin && $flow->navigator == 'primary' && common::hasPriv($flow->module, 'search')) $this->lang->{$flow->module}->menu->bysearch = "<a href='javascript:;' class='querybox-toggle'><i class='icon-search icon'></i> {$this->lang->searchLang}</a>";
            }

            /* The build in flows do not need to reset main menu. */
            if(!$flow->buildin)
            {
                if($flow->navigator == 'primary')
                {
                    /* Get default main menu order. */
                    $menuOrder = $this->lang->menuOrder;
                }
                else
                {
                    /* Get default module menu order. */
                    $menuOrder = isset($this->lang->{$flow->app}->menuOrder) ? $this->lang->{$flow->app}->menuOrder : $this->lang->menuOrder;
                }
                /* Reset menu order. */
                if(!empty($flow->position))
                {
                    $i = 1;
                    ksort($menuOrder);
                    foreach($menuOrder as $moduleName)
                    {
                        if('before' . $moduleName == $flow->position) array_splice($menuOrder, ($i - 1), 0, array('' => $flow->module));
                        if('after' . $moduleName == $flow->position)  array_splice($menuOrder, $i, 0, array('' => $flow->module));
                        $i++;
                    }
                }

                if($flow->navigator == 'primary')
                {
                    /* Reset main menu. */
                    $this->lang->menu      = $menu;
                    /* Reset main menu order. */
                    $this->lang->menuOrder = $menuOrder;

                    if($flow->module == $rawModule) $this->lang->flow->menu = $this->lang->{$flow->module}->menu;
                }
                else
                {
                    $this->lang->menugroup->flow = $flow->app;

                    /* Reset module menu. */
                    if(isset($this->lang->{$flow->app}->menu))
                    {
                        $this->lang->{$flow->app}->menu = $menu;

                        if($flow->module == $rawModule) $this->lang->flow->menu = $menu;
                    }
                    else
                    {
                        $this->lang->menu = $menu;
                    }

                    /* Reset module menu order. */
                    if(isset($this->lang->{$flow->app}->menuOrder))
                    {
                        $this->lang->{$flow->app}->menuOrder = $menuOrder;
                    }
                    else
                    {
                        $this->lang->menuOrder = $menuOrder;
                    }
                }
            }

            /* Reset feature bar. */
            if($flow->module == 'product')
            {
                $this->lang->product->featureBar['all'] = $featureBar;
            }
            elseif($flow->module == 'story')
            {
                $this->lang->product->featureBar['browse'] = $featureBar;
            }
            elseif($flow->module == 'project')
            {
                $this->lang->project->featureBar['all'] = $featureBar;
            }
            elseif($flow->module == 'task')
            {
                $this->lang->project->featureBar['task'] = $featureBar;
            }
            elseif($flow->module == 'bug')
            {
                $this->lang->bug->featureBar['browse'] = $featureBar;
            }
            else
            {
                if(isset($this->lang->{$flow->module}->featureBar['browse']))
                {
                    $this->lang->{$flow->module}->featureBar['browse'] = $featureBar;
                }
                else
                {
                    $this->lang->{$flow->module}->featureBar = $featureBar;
                }
            }

            $this->lang->{'flow_' . $flow->module} = $this->lang->{$flow->module};
        }

        if($this->app->isFlow)
        {
            if($flowApp == 'product' or $flowApp == 'qa')
            {
                $products  = $this->loadModel('product')->getPairs('nocode');
                $productID = $this->product->saveState(0, $products);
                $branch    = (int)$this->cookie->preBranch;
                if($flowApp == 'product') $this->product->setMenu($products, $productID, $branch);
                if($flowApp == 'qa') $this->loadModel('qa')->setMenu($products, $productID, $branch);
            }
            elseif($flowApp == 'project')
            {
                $projects  = $this->loadModel('project')->getPairs('nocode');
                $projectID = $this->project->saveState(0, $projects);
                $this->project->setMenu($projects, $projectID);
            }
        }
    }

    /**
     * Get default feature bar.
     *
     * @param  object $flow
     * @access public
     * @return bool | array | object
     */
    public function getFeatureBar($flow)
    {
        if($flow->navigator != 'secondary') return false;

        switch($flow->module)
        {
        case 'product' :
            $this->app->loadLang('product');
            return $this->lang->product->featureBar['all'];
        case 'story' :
            $this->app->loadLang('product');
            return $this->lang->product->featureBar['browse'];
        case 'project' :
            $this->app->loadLang('project');
            return $this->lang->project->featureBar['all'];
        case 'task' :
            $this->app->loadLang('project');
            return $this->lang->project->featureBar['task'];
        case 'bug' :
            $this->app->loadLang('bug');
            return $this->lang->bug->featureBar['browse'];
        default :
            $this->app->loadLang($flow->module);

            return !empty($this->lang->{$flow->module}->featureBar['browse']) ? $this->lang->{$flow->module}->featureBar['browse'] : $this->lang->{$flow->module}->featureBar;
        }
    }

    public function sortFlows($flows)
    {
        $flowGroups = array();
        $flowOrder  = array();
        foreach($flows as $flow) $flowGroups[$flow->navigator][$flow->app][$flow->module] = $flow;

        $i = 1;
        foreach($flowGroups as $navigator)
        {
            foreach($navigator as $apps)
            {
                foreach($apps as $flow)
                {
                    if(isset($flowOrder[$flow->id])) continue;

                    $flowOrder[$flow->id] = $i;
                    if(strpos($flow->position, 'before') !== false or strpos($flow->position, 'after') !== false)
                    {
                        $mode   = strpos($flow->position, 'before') !== false ? 'before' : 'after';
                        $module = str_replace($mode, '', $flow->position);
                        if(isset($apps[$module]))
                        {
                            $linkFlow = $apps[$module];
                            $flowOrder[$linkFlow->id] = $i - 0.5;
                        }
                    }
                    $i++;
                }
            }
        }
        asort($flowOrder);
        return array_keys($flowOrder);
    }
}
